import Vue from "vue";
import VueRouter from "vue-router";
import store from "@/index/store";
import { generateIndexRouter } from "./generateIndexRouter";
import systemSettings from './systemSettings'
import { accessTokenError } from "@/index/utils/errorModal";

// hack router push callback
const originalPush = VueRouter.prototype.push;
VueRouter.prototype.push = function push(location, onResolve, onReject) {
  if (onResolve || onReject) return originalPush.call(this, location, onResolve, onReject);
  return originalPush.call(this, location).catch((err) => err);
};

Vue.use(VueRouter);

const routes = [
  {
    path: "/404",
    component: () => import(/* webpackChunkName: "404" */ "@/index/views/404"),
  },
  {
    path: "/loginFailed",
    component: () => import( "@/index/views/LoginFailed"),
  },
  // todo 管理员才添加系统设置路由
  // systemSettings
];

const router = new VueRouter({
  routes,
  scrollBehavior: () => ({ y: 0 }), // 管理滚动行为 如果出现滚动 切换就让页面回到顶部
});

let dynamicRoutes = [];

router.beforeEach(async (to, from, next) => {
  const { token: queryToken } = to.query;
  const userState = store.state.user;
  // const storageToken = Vue.ls.get("token");
  const storageToken = Vue.ls.get("token")||'123test'; // 离线状态测试用

  // 1. 从 TAPD 入口进来, 更新token并重定向
  if (queryToken) return handleTapdLogin(queryToken, to, next);

  // 2. Vuex 和本地存储都没有 token，处理登录失败情况, 结束导航
  if (!userState.token && !storageToken) return handleNoToken(next);

  // 3. Vuex 没有 token，但 localStorage 有，恢复 token
  if (!userState.token && storageToken) restoreToken(storageToken);

  // 4. Vuex 有 token，但还没有动态路由，获取用户资源, 生成动态路由
  if (userState.token && dynamicRoutes.length === 0) loadRoutes(userState, next);

  // 5. 放行
  next();
});

/** 处理从 TAPD 入口进来的情况 */
async function handleTapdLogin(queryToken, to, next) {
  Vue.ls.remove("token"); // 清除本地 token

  if (await fetchToken(queryToken)) {
    Vue.ls.set("token", store.state.user.token);
    return next({ path: to.path, query: {} }); // 移除 URL 中的 token 参数
  }

  return handleLoginFailure(next);
}

/** 通过临时 token 获取正式 token */
async function fetchToken(tmpToken) {
  const formData = new FormData();
  formData.append("tmp_token", tmpToken);
  await store.dispatch("user/getToken", formData);
  return Boolean(store.state.user.token);
}

/** 处理没有 token 的情况 */
function handleNoToken(next) {
  showTokenError();
  next(false);
}

/** 从 localStorage 恢复 token */
function restoreToken(token) {
  store.commit("user/changeToken", token);
}

/** 生成并加载动态路由 */
async function loadRoutes(userState, next) {
  try {
    await store.dispatch('user/getUserInfo')
    const routesResource = await store.dispatch("user/getRoutesResource");
    dynamicRoutes = generateIndexRouter(routesResource);
    router.addRoutes(dynamicRoutes);
    if (userState.isAdmin) {
      router.addRoutes([systemSettings]);
    }
  } catch (err) {
    Vue.prototype.$error("请求用户资源失败");
    console.error(err);
    next(false);
  }
}

/** 处理登录失败情况 */
function handleLoginFailure(next) {
  return store.state.user.loginFailed ? next({ path: "/loginFailed" }) : showTokenError(next);
}

/** 显示 token 相关的错误提示 */
function showTokenError() {
  if (!Vue.prototype.$closeModalFun) {
    Vue.prototype.$closeModalFun = accessTokenError("获取 token 失败，请点击左边导航栏应用“资源管理”重试。");
  }
}

export default router;
